# Client Installation

## Source

Add all the source files to your Xcode project.

## Preprocessor defines

Add a preprocessor define to your build settings of *SERVER=@"http://your.serverurl.goeshere.com<:optional port number>"*

If you are NOT using the included PersistentStoreSingleton class, you MUST define `STAND_ALONE_SYNC` and you must also provide an object to the sync server which implements the CoreDataContextCreator protocol.  You do this using the method `+globalInstanceWithContextCreator:` which must be called **ONLY** once and **ONLY** before you start the sync server.

## A word about the interface.

Not all the methods/properties exposed in the header are actually public.  Use caution.

## Starting the sync server.

In your application delegate, in your `-applicationDidBecomeActive` method add:

    [[SyncController globalInstance] start];

## Saving sync state when your application goes inactive.  

And in your `-applicationWillResignActive` method:

    [[SyncController globalInstance] cancel];
    [[SyncController globalInstance] save];
    [[SyncController globalInstance] forceSync];

## Loading saved state when you restart.

In your `-application:didFinishLaunching` do:

    [[SyncController globalInstance] load];

## Configuring the username and password.

Somewhere in your app, you need to allow them to configure an email and password.  To create an account, do something like this:

    -(IBAction)signupTapped:(id)sender;
    {
        NSMutableDictionary *params = [NSMutableDictionary dictionary];
        [params setObject:[self.emailAddr text] forKey:@"email"];
        [params setObject:[self.password text] forKey:@"password"];
        [params setObject:[self.confirm text] forKey:@"password_confirmation"];
        
        NSMutableDictionary *requestParams = [NSMutableDictionary dictionaryWithObject:params forKey:@"user"];
        
        NSString *jsonData = [[CJSONSerializer serializer] serializeDictionary:requestParams error:nil];
        NSMutableURLRequest *request = [NSMutableURLRequest postRequestForURL:[NSURL URLWithString:[NSString stringWithFormat:@"%@/users.json", SERVER]] withJSON:jsonData];
        
        JSONWebCommand *signupCommand = [[JSONWebCommand alloc] initWithRequest:request];
    
        [signupCommand setCompletionBlock:^(NSDictionary *inResult)
         {
              // SUCCESS! Store the username and password!
             [[NSUserDefaults standardUserDefaults] setObject:[self.emailAddr text] forKey:EMAIL_ADDR];
             [[NSUserDefaults standardUserDefaults] setObject:[self.password text] forKey:PASSWORD];
             [[NSUserDefaults standardUserDefaults] synchronize];
  
         }];
        
        [signupCommand setFailureBlock:^(NSError *inError)
         {
             NSLog(@"%@", inError);
             [spinner dismiss];
             NSDictionary *userInfo = [inError userInfo];
             
             NSString *errorMsg = [inError localizedDescription];
             
             if([userInfo objectForKey:@"error"])
             {
                 NSDictionary *errorInfo = [userInfo objectForKey:@"error"];
                 errorMsg = [NSString stringWithFormat:@"%@ - %@", [[errorInfo allKeys] objectAtIndex:0], [errorInfo objectForKey:[[errorInfo allKeys] objectAtIndex:0]]];
             }

              // FAILURE - display the error message...
             
         }];
    
        [signupCommand start];
    }

To verify a username and password (ie: when they are logging in - if they already have an account) do:

    -(IBAction)loginTapped:(id)sender;
    {
        NSMutableURLRequest *request = [NSMutableURLRequest requestWithURL:[NSURL URLWithString:[NSString stringWithFormat:@"%@/users/verify.json", SERVER]]];
        [request addAuthenticationForUsername:[self.emailAddr text] password:[self.password text]];
        
        JSONWebCommand *command = [[JSONWebCommand alloc] initWithRequest:request];
                
      [command setCompletionBlock:^(NSDictionary *inResult)
       {
             [[NSUserDefaults standardUserDefaults] setObject:[self.emailAddr text] forKey:EMAIL_ADDR];
             [[NSUserDefaults standardUserDefaults] setObject:[self.password text] forKey:PASSWORD];
             [[NSUserDefaults standardUserDefaults] synchronize];
       }];
        
        [command setFailureBlock:^(NSError *inError)
         {
             NSDictionary *userInfo = [inError userInfo];
    
             DismissableAlertView *alert = [[DismissableAlertView alloc] initWithTitle:@"Account Error" message:[NSString stringWithFormat:@"An error occurred while logging in: %@", [userInfo objectForKey:@"error"]] delegate:nil cancelButtonTitle:@"OK" otherButtonTitles:nil];
             [alert show];
         }];
        
        [command start];
    }

## Forcing a sync and being called back...

Sometimes you want to force a sync and then be informed when the sync has completed... (for example, to know that the latest data has been loaded) To do this, use the method: `-forceSyncWithContinuation:` which takes a block of code which will be executed on your main thread when the sync is complete.  Only one continuation can be active at any time.


